@@ -47,7 +47,7 @@ Entry::Entry( Date date, const Glib::ustring &text )
 {
 }
 
-const std::string
+std::string
 Entry::get_date_str( void ) const
 {
 	std::ostringstream oss;
@@ -117,8 +117,9 @@ Entry::get_tags( void )
 
 // DATABASE
 Database::Database()
-	:	m_flag_encrypted( false ), m_option_spellcheck( false ),
-	m_option_entryliststyle( 0 ), m_theme( NULL ), m_flag_old( false )
+	:	m_passphrase( "" ), m_filter( "" ), m_flag_encrypted( false ),
+	m_option_spellcheck( false ), m_option_entryliststyle( 0 ),
+	m_theme( NULL ), m_flag_old( false )
 {
 
 }
@@ -146,7 +147,7 @@ void
 Database::set_passphrase( const std::string &passphrase )
 {
 	// TODO: resetting the password should be carried out by a dedicated function
-	if( passphrase.size() >= PASSPHRASE_MIN_SIZE || passphrase.size() == 0 )
+	if( passphrase.size() >= PASSPHRASE_MIN_SIZE )
 	{
 		m_passphrase = passphrase;
 		m_flag_encrypted = true;
@@ -724,17 +725,19 @@ Database::write_txt( const std::string &path )
 			file << " ***";
 
 		// TAGS
-		if( ( *itr_entry )->get_tags().size() > 0 )
+		Tagset *tagset = &( ( *itr_entry )->get_tags() );
+		for(	Tag::Tagpool::const_iterator itr_tags = tagset->begin();
+				itr_tags != tagset->end();
+				itr_tags++ )
 		{
-			Tag::Tagpool::const_iterator itr_tags = Tag::get_pool().begin();
-			file << "\n>>> " << ( *itr_tags )->get_name();
-			itr_tags++;
-			for( ; itr_tags != Tag::get_pool().end(); itr_tags++ )
-			{	
-				if( ( *itr_entry )->get_tags().checkfor_member( *itr_tags ) )
-					file << ", "<< ( *itr_tags )->get_name();
-			}
+			if( itr_tags == tagset->begin() )
+				file << "\n>>> "; // FIXME: use a more meaningful sign
+			else
+				file << ", ";
+
+			file << ( *itr_tags )->get_name();
 		}
+
 		// CONTENT
 		file << "\n\n" << ( *itr_entry )->get_text();
 
@@ -820,7 +823,8 @@ Database::clear()
 	m_entries.clear();
 	Tag::clear_pool();
 	m_chaptersets.clear();
-	m_passphrase = "";
+	m_passphrase.clear();
+	m_filter.clear();
 	// NOTE: only reset body options here:
 	m_option_spellcheck = true;
 	m_option_entryliststyle = 0;
@@ -831,6 +835,15 @@ Database::clear()
 	}
 }
 
+void
+Database::delete_tag( Tag *tag )
+{
+	for( Entryiter iter = m_entries.begin(); iter != m_entries.end(); iter++ )
+		( * iter )->get_tags().remove_tag( tag );
+
+	Tag::delete_tag( tag );
+}
+
 Entry*
 Database::get_entry_today( void )
 {
@@ -866,26 +879,40 @@ Database::get_entry( Date date )
 }
 
 Entry*
-Database::get_entry_create( Date date )
+Database::get_entry( Date date, int dayorder )
 {
-	Entry *entry = get_entry( date );
-
-	if( entry )
-		return entry;
-	else
-		return create_entry( date );
+	int i = 0;
+	for( Entryiter iter = m_entries.begin(); iter != m_entries.end(); iter++ )
+	{
+		if( ( *iter )->get_filtered_out() == false )
+			if( ( *iter )->get_date() == date )
+			{
+				if( i == dayorder )
+					return( *iter );
+				else
+				if( i < dayorder )
+					i++;
+				else
+					break;
+			}
+	}
+	return NULL;
 }
 
-const Entry*
-Database::get_entry( int entry ) const
+Entryvector*
+Database::get_entries( Date date )
 {
-	return m_entries[ entry ];
-}
+	Entryvector *entries = new Entryvector;
 
-Entry*
-Database::get_entry( int entry )
-{
-	return m_entries[ entry ];
+	for( Entryiter iter = m_entries.begin(); iter != m_entries.end(); iter++ )
+	{
+		if( ( *iter )->get_filtered_out() == false )
+			if( ( *iter )->get_date() == date )
+			{
+				entries->push_back( *iter );
+			}
+	}
+	return entries;
 }
 
 Entry*
@@ -933,10 +960,58 @@ Database::get_entry_first( void )
 	return NULL;
 }
 
+Entry*
+Database::get_entry_nextinday( Date date, const Entry *entry )
+{
+	Entryiter	iter_last			= m_entries.end();
+	Entryiter	iter_firstinday		= iter_last;
+	bool		flag_found			= false;
+
+	for( Entryiter iter = m_entries.begin(); iter != iter_last; iter++ )
+	{
+		if( ( *iter )->get_date() == date )
+		{
+			if( iter_firstinday == iter_last )
+				iter_firstinday = iter;
+			if( *iter == entry )
+				flag_found = true;
+			else
+			if( flag_found )
+				return( *iter );
+		}
+		else
+		if( iter_firstinday != iter_last )
+			break;
+	}
+
+	if( iter_firstinday != iter_last && *iter_firstinday != entry )
+		return( * iter_firstinday );
+	else
+		return NULL;
+}
+
+int
+Database::get_entrydayorder( Entry const*const entry ) const
+{
+	int dayorder = 0;
+	for( Entryciter iter = m_entries.begin(); iter != m_entries.end(); iter++ )
+		if( entry->get_date() == ( *iter )->get_date() )
+		{
+			if( *iter == entry )
+			{
+				return dayorder;
+			}
+			else
+				dayorder++;
+		}
+
+	return -1;
+}
+
 bool
 Database::is_first( Entry const*const entry ) const
 {
-	return( entry == m_entries[ 0 ] );
+	return( entry == m_entries.front() );
 }
 
 bool
@@ -1001,11 +1076,13 @@ Database::set_filter( const Glib::ustring &string )
 			std::string::npos );
 	}
 
+	m_filter = string;
+
 	return 0;   // reserved
 }
 
 void
-Database::remove_filter()
+Database::remove_filter( void )
 {
 	Entryiter itr_entry;
 
@@ -1013,6 +1090,45 @@ Database::remove_filter()
 	{
 		( *itr_entry )->set_filtered_out( false );
 	}
+
+	m_filter.clear();
+}
+
+bool
+Database::is_filter_set( void )
+{
+	return( ! m_filter.empty() );
+}
+
+int
+Database::replace_filter( const Glib::ustring& newtext )
+{
+	Glib::ustring::size_type iter_str;
+	const int chardiff = newtext.size() - m_filter.size();
+
+	for(	Entryiter iter_entry = m_entries.begin();
+			iter_entry != m_entries.end();
+			iter_entry++ )
+	{
+		if( ( *iter_entry )->get_filtered_out() )
+			continue;
+
+		Glib::ustring entrytext = ( *iter_entry )->get_text().lowercase();
+		iter_str = 0;
+		int count = 0;
+		while(	( iter_str = entrytext.find( m_filter, iter_str ) ) !=
+				std::string::npos )
+		{
+			( *iter_entry )->get_text().erase(
+				iter_str + ( chardiff * count ), m_filter.size() );
+			( *iter_entry )->get_text().insert(
+				iter_str + ( chardiff * count ), newtext );
+			count++;
+			iter_str += m_filter.size();
+		}
+	}
+	
+	return 0;	// reserved
 }
 
 Chapterset*
@@ -1069,61 +1185,58 @@ Database::find_uniquename_chapterset( const Glib::ustring &name )
 
 // TAG
 // static member declarations
-Tag::Tagpool Tag::Pool;
+Tag::Tagpool Tag::m_pool;
 
-Tag::Tag( const std::string &name, unsigned int userCount )
-	: Name( name ), UserCount( userCount )
+Tag::Tag( const Glib::ustring &name, unsigned int usercount )
+	: m_name( name ), m_usercount( usercount )
 {
-	Pool.push_back( this );
+	m_pool.push_back( this );
 }
 
 Tag::~Tag( void )
 {
-	Pool.remove( this );
+	m_pool.remove( this );
 }
 
 Tag*
-Tag::create( const std::string &name )
+Tag::create( const Glib::ustring &name )
 {
-	for(    Tag::Tagpool::iterator iter = Pool.begin();
-			iter != Pool.end();
+	for(    Tag::Tagpool::iterator iter = m_pool.begin();
+			iter != m_pool.end();
 			iter++ )
 	{
-		if( name == ( *iter )->Name )
-		{
-			( *iter )->UserCount++;
+		if( name == ( *iter )->m_name )
 			return( *iter );
-		}
 	}
-	return new Tag( name, 1 );
+	return new Tag( name, 0 );
 }
 
 void
 Tag::delete_tag( Tag *tag )
 {
-	Pool.remove( tag );
+	m_pool.remove( tag );
 	delete tag;
 }
 
 Tag*
 Tag::get_tag( unsigned int tagorder )
 {
-	if( tagorder >= Pool.size() )
+	if( tagorder >= m_pool.size() )
 		return NULL;
 
-	Tag::Tagpool::const_iterator iter = Pool.begin();
+	Tag::Tagpool::const_iterator iter = m_pool.begin();
 	advance( iter, tagorder );
 	return( *iter );
 }
 
 Tag*
-Tag::get_tag( const std::string &name )
+Tag::get_tag( const Glib::ustring &name )
 {
-	for(    Tag::Tagpool::iterator iter = Pool.begin();
-			iter != Pool.end();
+	for(    Tag::Tagpool::iterator iter = m_pool.begin();
+			iter != m_pool.end();
 			iter++ )
 	{
-		if( name == ( *iter )->Name )
+		if( name == ( *iter )->m_name )
 		{
 			return( *iter );
 		}
@@ -1134,31 +1247,25 @@ Tag::get_tag( const std::string &name )
 const Tag::Tagpool&
 Tag::get_pool( void )
 {
-	return Pool;
+	return m_pool;
 }
 
 void
 Tag::clear_pool( void )
 {
-	Pool.clear();
+	m_pool.clear();
 }
 
-unsigned int
-Tag::decrease_usercount( unsigned int difference )
-{
-	return UserCount -= difference;
-}
-
-const std::string&
+const Glib::ustring&
 Tag::get_name( void ) const
 {
-	return Name;
+	return m_name;
 }
 
 void
-Tag::set_name( const std::string &name )
+Tag::set_name( const Glib::ustring &name )
 {
-	Name = name;
+	m_name = name;
 }
 
 // TAGSET
@@ -1168,12 +1275,12 @@ Tagset::~Tagset( void )
 			iter != this->end();
 			iter++ )
 	{
-		( *iter )->decrease_usercount();
+		( *iter )->m_usercount--;
 	}
 }
 
 bool
-Tagset::add_tag( const std::string &name )
+Tagset::add_tag( const Glib::ustring &name )
 {
 	Tag *tag;
 
@@ -1193,11 +1300,10 @@ Tagset::add_tag( const std::string &name )
 			iter++ )
 	{
 		if( tag == ( *iter ) )
-		{
 			return false;
-		}
 	}
 
+	tag->m_usercount++;
 	this->push_back( tag );
 	return true;
 }
@@ -1215,6 +1321,7 @@ Tagset::add_tag( Tag *tag )
 		}
 	}
 
+	tag->m_usercount++;
 	this->push_back( tag );
 	return true;
 }
@@ -1253,7 +1360,7 @@ Tagset::get_tag( unsigned int tagorder ) const
 }
 
 bool
-Tagset::remove_tag( const std::string &name )
+Tagset::remove_tag( const Glib::ustring &name )
 {
 	for(
 		Tag::Tagpool::iterator iter = this->begin();
@@ -1262,7 +1369,7 @@ Tagset::remove_tag( const std::string &name )
 	{
 		if( name == ( *iter )->get_name() )
 		{
-			( *iter )->decrease_usercount();
+			( *iter )->m_usercount--;
 			this->remove( *iter );
 			return true;
 		}
@@ -1274,7 +1381,10 @@ void
 Tagset::remove_tag( Tag *ptr2tag )
 {
 	if( ptr2tag != NULL )
+	{
+		ptr2tag->m_usercount--;
 		remove( ptr2tag );
+	}
 }
 
 // CHAPTERS
