@@ -367,6 +367,15 @@ EntryIdletext::on_focus_in_event( GdkEventFocus *event )
 	return( Gtk::Entry::on_focus_in_event( event ) );
 }
 
+bool
+EntryIdletext::on_key_release_event( GdkEventKey *event )
+{
+	if( event->keyval == GDK_Escape )
+		set_text( "" );
+		
+	return Gtk::Entry::on_key_release_event( event );
+}
+
 // MENUBUTTON
 Menubutton::Menubutton(	const Gtk::StockID &stockid,
 						const Glib::ustring &label,
@@ -515,7 +524,7 @@ void Menutoolbutton::on_toolbar_reconfigured (void)
 // MENUITEMRECENT
 MenuitemRecent::MenuitemRecent( const std::string &path )
 	:	Gtk::MenuItem(), m_icon_remove( Gtk::Stock::DELETE, Gtk::ICON_SIZE_MENU ),
-	m_path( path )
+	m_path( path ), m_flag_deletepressed( false )
 {
 	Gtk::Label		*label = Gtk::manage( new Gtk::Label(
 											Glib::filename_display_basename( path ) ) );
@@ -539,7 +548,9 @@ MenuitemRecent::MenuitemRecent( const std::string &path )
 bool
 MenuitemRecent::on_motion_notify_event( GdkEventMotion *event )
 {
-	m_icon_remove.show();
+	if( !( event->state &
+			( Gdk::BUTTON1_MASK | Gdk::BUTTON2_MASK | Gdk::BUTTON3_MASK ) ) )
+		m_icon_remove.show();
 
 	return Gtk::MenuItem::on_motion_notify_event( event );
 }
@@ -553,19 +564,35 @@ MenuitemRecent::on_leave_notify_event( GdkEventCrossing *event )
 }
 
 bool
-MenuitemRecent::on_button_release_event( GdkEventButton *event )
+MenuitemRecent::on_button_press_event( GdkEventButton *event )
 {
 	if(	event->x >= m_icon_remove.get_allocation().get_x() &&
 		event->x < m_icon_remove.get_allocation().get_x() +
 					m_icon_remove.get_allocation().get_width() )
-	{
-		m_signal_removerecent.emit( m_path );
-		return true;
-	}
+		m_flag_deletepressed = true;
+	else
+		m_flag_deletepressed = false;
 
 	return Gtk::MenuItem::on_button_press_event( event );
 }
 
+bool
+MenuitemRecent::on_button_release_event( GdkEventButton *event )
+{
+	if( m_flag_deletepressed )
+	{
+		if(	event->x >= m_icon_remove.get_allocation().get_x() &&
+			event->x < m_icon_remove.get_allocation().get_x() +
+						m_icon_remove.get_allocation().get_width() )
+		{
+			m_signal_removerecent.emit( m_path );
+			return true;
+		}
+	}
+
+	return Gtk::MenuItem::on_button_release_event( event );
+}
+
 
 
 // FILEBUTTONRECENT
@@ -727,9 +754,13 @@ FilebuttonRecent::show_filechooser( void )
 {
 	m_filechooserdialog = new Gtk::FileChooserDialog(
 	// TRANSLATORS: this is the title of filechooser dialog for selecting a new diary
-			_("Select a Diary"), Gtk::FILE_CHOOSER_ACTION_OPEN );
-	m_filechooserdialog->set_filename( m_list_recent.size() > 0 ?
-										*m_list_recent.begin() : Glib::get_home_dir() );
+			_( "Select a Diary" ), Gtk::FILE_CHOOSER_ACTION_OPEN );
+
+	if( m_list_recent.size() > 0 )
+		m_filechooserdialog->set_filename( * m_list_recent.begin() );
+	else
+		m_filechooserdialog->set_current_folder( Glib::get_home_dir() );
+
 	m_filechooserdialog->add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
 	m_filechooserdialog->add_button( Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT );
 	FilefilterAny		filter_any;
