@@ -122,13 +122,14 @@ class Error
 class EntryIdletext : public Gtk::Entry
 {
 	public:
-							EntryIdletext( const Glib::ustring& );
+		explicit			EntryIdletext( const Glib::ustring& );
 
 	protected:
 		virtual void		on_map( void );
 		virtual void		on_changed( void );
 		virtual bool		on_expose_event( GdkEventExpose* );
 		virtual bool		on_focus_in_event( GdkEventFocus* );
+		virtual bool		on_key_release_event( GdkEventKey* );
 
 		Glib::ustring		m_idletext;
 		bool				m_flag_empty;
@@ -156,7 +157,7 @@ class Menubutton : public Gtk::ToggleButton
 							Menubutton(	const std::string&,
 										const Glib::ustring&,
 										Gtk::ReliefStyle = Gtk::RELIEF_NONE );
-							~Menubutton( void );
+		virtual				~Menubutton( void );
 
 		Menu2*&				get_menu( void );
 		void				release( void );
@@ -227,9 +228,11 @@ class MenuitemRecent : public Gtk::MenuItem
 		virtual bool				on_motion_notify_event( GdkEventMotion* );
 		virtual bool				on_leave_notify_event( GdkEventCrossing* );
 		virtual bool				on_button_release_event( GdkEventButton* );
+		virtual bool				on_button_press_event( GdkEventButton* );
 
 		Gtk::Image					m_icon_remove;
 		std::string					m_path;
+		bool						m_flag_deletepressed;
 
 		Signal_void_string			m_signal_removerecent;
 };
