@@ -29,37 +29,25 @@
 
 using namespace LIFEO;
 
-namespace LIFEO
-{
-//	extern ColrecTag *colrec_tag;
-}
-
 
 // CONSTRUCTOR
 Lifeograph::Lifeograph ( int argc, char *argv[] )
 	:	m_dialog_password( NULL ), m_dialog_tageditor( NULL ),
-	m_dialog_chaptereditor( NULL ), m_dialog_export( NULL ),
-	m_textbufferdiary( NULL ), m_menu_treeviewentries( NULL ), m_menu_tag( NULL ),
+	m_dialog_chaptereditor( NULL ), m_dialog_export( NULL ), m_textbufferdiary( NULL ),
+	m_menu_treeviewentries( NULL ), m_menu_tag( NULL ), m_menu_link( NULL ),
 	m_programpath( argv[ 0 ] ), m_loginstatus( LOGGED_OUT ), m_option_idletime_max( 90 ),
 	m_option_autologout( true ), m_option_bg_path( "" ), m_entrycount( 0 ),
 	m_internaloperation( 0 ), m_flag_entrychanged( false ),
 	m_ptr2listitem_cur( NULL ), m_ptr2listitem_prev( NULL )
 {
-	m_filebuttonrecent = new LIFEO::FilebuttonRecent( _( "Select a Diary File" ) );
+	m_filebuttonrecent = new LIFEO::FilebuttonRecent( _( "Select or Create a Diary" ) );
+	m_textbufferdiary = new TextbufferDiary( &m_undomanager, &m_database );
 	m_window = new Gtk::Window;
-	colrec_tag = new ColrecTag;	// FIXME: this is ridiculous!
+	colrec_tag = new ColrecTag;	// global variable
 
+	// CONFIGURATION
 	m_gconfclient = Gnome::Conf::Client::get_default_client();
-
-	m_filebuttonrecent->signal_selection_changed().connect(
-			sigc::mem_fun( *this, &Lifeograph::handle_filebutton_db_selectionchanged ) );
-	m_filebuttonrecent->signal_create_file().connect(
-			sigc::mem_fun( *this, &Lifeograph::handle_create_new_diary ) );
-	m_window->signal_delete_event().connect(
-			sigc::mem_fun( *this, &Lifeograph::handle_event_delete ) );
-
-	Gtk::AboutDialog::set_url_hook( &open_url );
-	Gtk::AboutDialog::set_email_hook( &mail_to );
+	m_flag_firsttimeuser = ! read_conf();
 
 	// COMMANDLINE OPTIONS
 	bool  force_welcome( false );
@@ -70,7 +58,7 @@ Lifeograph::Lifeograph ( int argc, char *argv[] )
 			if( i < argc )
 			{
 				if( Glib::file_test( argv[ ++i ], Glib::FILE_TEST_EXISTS ) )
-					m_database.set_path( argv[ i ] );
+					m_filebuttonrecent->add_recent( argv[ i ] );
 			}
 			else
 				Error( "No path provided" );
@@ -97,6 +85,12 @@ Lifeograph::Lifeograph ( int argc, char *argv[] )
 
 				std::cout << "Purged gconf keys related to Lifeograph." << std::endl;
 			}
+			else
+				std::cout << "No gconf key found." << std::endl;
+
+			delete m_window;
+			m_window = NULL;
+			return;
 		}
 		else
 		if( ! strcmp( argv[ i ], "--set-bg" ) )
@@ -114,10 +108,14 @@ Lifeograph::Lifeograph ( int argc, char *argv[] )
 		else
 		if( Glib::file_test( argv[ i ], Glib::FILE_TEST_EXISTS ) )
 		{
-			m_database.set_path( argv[ i ] );
+			m_filebuttonrecent->add_recent( argv[ i ] );
 		}
 	}
 
+	// PROGRAM-WIDE VARIABLE INITIALIZATIONS
+	Gtk::AboutDialog::set_url_hook( &open_url );
+	Gtk::AboutDialog::set_email_hook( &mail_to );
+
 	m_treestore_entries = Gtk::TreeStore::create( m_colrec_entry );
 	m_liststore_tagtable = Gtk::ListStore::create( *colrec_tag );
 	m_liststore_tagcombo = Gtk::ListStore::create( *colrec_tag );
@@ -127,16 +125,59 @@ Lifeograph::Lifeograph ( int argc, char *argv[] )
 	m_treestore_entries->set_sort_func(
 			m_colrec_entry.m_info, sigc::mem_fun( *this, &Lifeograph::sort_bytitle ) );
 
-	if( read_conf() && ! force_welcome )
-		draw_loginscreen();
-	else // first time user
+	// BACKGROUND PICTURE
+	if( m_option_bg_path == "none" )	// disable bg picture
+		;
+	else
+	if( ! Glib::file_test( m_option_bg_path, Glib::FILE_TEST_EXISTS ) )
+	{
+		// try to use the default bg picture:
+		m_option_bg_path = PIXMAPDIR "/lifeograph/backgrounds/bg.png";
+		if( ! Glib::file_test( m_option_bg_path, Glib::FILE_TEST_EXISTS ) )
+		{
+			Error error1( "No background!" );
+			m_option_bg_path.clear();
+		}
+	}
+	if( ! m_option_bg_path.empty() && m_option_bg_path != "none" )
+		m_pixbuf_background = Gdk::Pixbuf::create_from_file( m_option_bg_path );
+
+	// RECENT FILES
+	if( m_filebuttonrecent->get_recentcount() > 0 )
+	{
+		m_database.set_path( m_filebuttonrecent->get_filename() );
+		// update label and tooltip:
+		m_filebuttonrecent->update_filename();
+	}
+
+	// INTERFACE INITIALIZATION
+	if( m_flag_firsttimeuser || force_welcome )
 		draw_welcomescreen();
+	else
+		draw_loginscreen();
+	
+	// SIGNALS
+	LinkEntry::signal_activated().connect(
+		sigc::mem_fun( *this, &Lifeograph::handle_textview_link_activated ) );
+	m_filebuttonrecent->signal_selection_changed().connect(
+			sigc::mem_fun( *this, &Lifeograph::handle_filebutton_db_selectionchanged ) );
+	m_filebuttonrecent->signal_create_file().connect(
+			sigc::mem_fun( *this, &Lifeograph::handle_create_new_diary ) );
+	m_textbufferdiary->signal_changed().connect(
+		sigc::mem_fun( *this, &Lifeograph::handle_textview_entry_changed ) );
+	m_textbufferdiary->signal_link_needs_checking().connect(
+		sigc::mem_fun( *this, &Lifeograph::handle_textbuffer_link_needs_checking ) );
+	m_textbufferdiary->signal_title_changed().connect(
+		sigc::mem_fun( *this, &Lifeograph::handle_textbuffer_title_changed ) );
+	m_window->signal_delete_event().connect(
+			sigc::mem_fun( *this, &Lifeograph::handle_event_delete ) );
 }
 
 void
 Lifeograph::run( void )
 {
-	Gtk::Main::run( *m_window );
+	if( m_window )
+		Gtk::Main::run( *m_window );
 }
 
 bool
@@ -157,24 +198,14 @@ Lifeograph::read_conf( void )
 			for( int i = LIFEO::MAX_RECENT_FILE_COUNT - 1; i >= 0; i-- )
 			{
 				string =
-					m_gconfclient->get_string(
-						Glib::ustring::compose( "%1%2%3",
-												CONFIG_PATH,
-												"/recent/",
-												i ) );
+						m_gconfclient->get_string(
+								Glib::ustring::compose(	"%1/recent/%2",
+														CONFIG_PATH,
+														i ) );
 				if( string.size() > 0 )
 					m_filebuttonrecent->add_recent( string );
 			}
 		}
-		if( m_database.is_path_set() )   // if set in commandline
-			m_filebuttonrecent->set_filename( m_database.get_path() );
-		else
-		if( m_filebuttonrecent->get_recentcount() > 0 )
-		{
-			m_database.set_path( m_filebuttonrecent->get_filename() );
-			// update label and tooltip:
-			m_filebuttonrecent->update_filename();
-		}
 
 		// WINDOW GEOMETRY
 		int1 = m_gconfclient->get_int( CONFIG_PATH + "/width" );
@@ -190,9 +221,8 @@ Lifeograph::read_conf( void )
 			m_window->move( int1, int2 );
 
 		// BACKGROUND PICTURE
-		if( m_option_bg_path.empty() )    // if not set in commandline
-			m_option_bg_path = m_gconfclient->get_string(
-										CONFIG_PATH + "/bg_picture_welcome" );
+		m_option_bg_path = m_gconfclient->get_string(
+				CONFIG_PATH + "/bg_picture_welcome" );
 
 		// IDLE TIME
 		int1 = m_gconfclient->get_int( CONFIG_PATH + "/max_idletime" );
@@ -205,29 +235,9 @@ Lifeograph::read_conf( void )
 	}
 	else
 	{
-		std::cout << "first time user!" << std::endl;
 		result = false;
 	}
 
-	// APPLY OPTIONS
-	// TODO: only background?
-	// BACKGROUND PICTURE
-	if( m_option_bg_path == "none" )	// disable bg picture
-		;
-	else
-	if( ! Glib::file_test( m_option_bg_path, Glib::FILE_TEST_EXISTS ) )
-	{
-		// try to use the default bg picture:
-		m_option_bg_path = PIXMAPDIR "/lifeograph/backgrounds/bg.png";
-		if( ! Glib::file_test( m_option_bg_path, Glib::FILE_TEST_EXISTS ) )
-		{
-			Error error1( "no background!" );
-			m_option_bg_path.clear();
-		}
-	}
-	if( ! m_option_bg_path.empty() && m_option_bg_path != "none")
-		m_pixbuf_background = Gdk::Pixbuf::create_from_file( m_option_bg_path );
-
 	return result;
 }
 
@@ -252,10 +262,7 @@ Lifeograph::write_conf( void )
 			m_filebuttonrecent->get_recentlist().begin();
 	for( int i = 0; i < m_filebuttonrecent->get_recentcount(); i++ )
 	{
-		m_gconfclient->set(	Glib::ustring::compose(	"%1%2%3",
-													CONFIG_PATH,
-													"/recent/",
-													i ),
+		m_gconfclient->set(	Glib::ustring::compose( "%1/recent/%2", CONFIG_PATH, i ),
 							*iter );
 		iter++;
 	}
@@ -405,11 +412,15 @@ Lifeograph::handle_button_about_clicked( void )
 	{
 		aboutdialog->set_logo( Gdk::Pixbuf::create_from_file( m_programpath + ".svg" ) );
 	}
-	aboutdialog->set_copyright( _( "Copyright (C) 2007-2009 Ahmet Öztürk" ) );
+	aboutdialog->set_copyright( _( "Copyright (C) 2007-2010 Ahmet Öztürk" ) );
 	aboutdialog->set_comments( _( STRING::SLOGAN ) );
-	aboutdialog->set_website( "http://blhps.dizayn.de/" );
-	aboutdialog->set_website_label( _( "Author's Homepage" ) );
+	aboutdialog->set_website( "https://launchpad.net/lifeograph/" );
+	aboutdialog->set_website_label( _( "Launchpad Page" ) );
 	aboutdialog->set_authors( authors );
+	aboutdialog->set_license(
+			"Lifeograph is licensed under GNU Public License v3" "\n\n"
+			"You should have received a copy of the GNU General Public License\n"
+			"along with Lifeograph.  If not, see <http://www.gnu.org/licenses/>" );
 	aboutdialog->set_transient_for( *m_window );
 	aboutdialog->run();
 	delete aboutdialog;
@@ -486,7 +497,6 @@ Lifeograph::handle_button_opendb_clicked( void )
 			draw_editscreen();
 			update_treeview_entries();
 			show_row( m_row_cur, UI_BOTH );
-			check_db_emptiness();
 			m_loginstatus = LOGGED_IN;
 			break;
 		case WRONG_PASSWORD:
@@ -509,20 +519,153 @@ Lifeograph::handle_button_opendb_clicked( void )
 void
 Lifeograph::handle_toolbutton_previous( void )
 {
-	if( m_row_cur != ( *m_row_cur.parent() ).children().begin() )
+	Gtk::TreeRow row_prev = m_row_cur;
+	if( m_row_cur[ m_colrec_entry.m_type ] == ColrecEntry::IT_Entry )
 	{
-		show_row( *( --m_row_cur ), UI_BOTH );
+		if( row_prev == ( *m_row_cur.parent() ).children().begin() )
+		{
+			row_prev = *( m_row_cur.parent() );
+			if( m_treestore_entries->children().size() > 1 )
+			{
+				do
+				{
+					if( row_prev == m_treestore_entries->children().begin() )
+					{
+						row_prev = *( m_treestore_entries->children().end() );
+					}
+					row_prev--;
+				}
+				while( row_prev.children().size() < 1 );
+			}
+	
+			row_prev = *( row_prev.children().end() );
+		}
+		row_prev--;
 	}
+	else
+//	if( m_row_cur[ m_colrec_entry.m_type ] == IT_Folder )
+	{
+		if( row_prev == m_treestore_entries->children().begin() )
+		{
+			row_prev = *( m_treestore_entries->children().end() );
+		}
+		row_prev--;
+	}
+
+	show_row( row_prev, UI_BOTH );
 }
 
 void
 Lifeograph::handle_toolbutton_next( void )
 {
 	Gtk::TreeRow row_next = m_row_cur;
-	if( ++row_next != ( *m_row_cur.parent() ).children().end() )
+	if( m_row_cur[ m_colrec_entry.m_type ] == ColrecEntry::IT_Entry )
 	{
-		show_row( row_next, UI_BOTH );
+		if( ++row_next == ( *m_row_cur.parent() ).children().end() )
+		{
+			row_next = *( m_row_cur.parent() );
+			if( m_treestore_entries->children().size() > 1 )
+			{
+				do
+				{
+					if( ++row_next == m_treestore_entries->children().end() )
+						row_next = *( m_treestore_entries->children().begin() );
+				}
+				while( row_next.children().size() < 1 );
+			}
+	
+			row_next = *( row_next.children().begin() );
+		}
 	}
+	else
+//	if( m_row_cur[ m_colrec_entry.m_type ] == IT_Folder )
+	{
+		if( ++row_next == m_treestore_entries->children().end() )
+			row_next = *( m_treestore_entries->children().begin() );
+	}
+
+	show_row( row_next, UI_BOTH );
+}
+
+void
+Lifeograph::handle_entry_filter_changed( void )
+{
+	if( m_internaloperation ) return;
+	// current entry must be closed here or else it loses...
+	// ...changes since it was selected:
+	sync_entry_current();
+
+	const std::string filterstring( m_entry_filter->get_text().lowercase() );
+	if( filterstring.size() > 0 )
+	{
+		m_database.set_filter( filterstring );
+		m_entry_filter->set_icon_sensitive( Gtk::ENTRY_ICON_SECONDARY, true );
+		m_textbufferdiary->set_searchstr( filterstring );
+		update_treeview_entries();
+		m_entry_filter->set_tooltip_markup(
+				Glib::ustring::compose(
+						_( "Found in <b>%1</b> of <b>%2</b> entrie(s)" ),
+						m_entrycount, m_database.get_size() ) );
+	}
+	else
+	{
+		m_database.remove_filter();
+		m_entry_filter->set_icon_sensitive( Gtk::ENTRY_ICON_SECONDARY, false );
+		m_textbufferdiary->set_search( false );
+		update_treeview_entries();
+		m_entry_filter->set_tooltip_text( _( "Enter text to be filtered" ) );
+	}
+
+	if( m_entrycount > 0 )
+	{
+		if( m_ptr2listitem_cur != m_textbufferdiary->get_entry() )
+		{
+			show_row( m_row_cur, UI_BOTH );
+			m_iconview_tags->show();
+		}
+		else
+		{
+			present_entryrow( m_row_cur );
+			m_textbufferdiary->reparse();
+		}
+	}
+	else
+	{
+		m_iconview_tags->hide();
+		m_textviewdiary->set_sensitive( false );
+
+		if( filterstring.size() > 0 )
+		{
+			m_internaloperation++;
+			m_textbufferdiary->set_richtext( _( "No matches found" ) );
+			m_hbox_tagtools->hide();
+			m_internaloperation--;
+		}
+	}
+
+	if( filterstring.size() > 0 && m_entrycount > 0 )
+	{
+		m_toolbutton_findprevious->show();
+		m_toolbutton_findnext->show();
+		m_hbox_replace->show();
+
+		m_textbufferdiary->select_searchstr_next();
+	}
+	else
+	{
+		m_toolbutton_findprevious->hide();
+		m_toolbutton_findnext->hide();
+		m_hbox_replace->hide();
+	}
+
+	update_calendar();
+}
+
+void
+Lifeograph::handle_entry_filter_clear( Gtk::EntryIconPosition,
+									   const GdkEventButton* )
+{
+	m_entry_filter->set_text( "" );
 }
 
 void
@@ -531,14 +674,17 @@ Lifeograph::handle_matchprevious( void )
 	if( m_entrycount < 1 )
 		return;
 
-	if( ! m_textbufferdiary->select_searchword_previous() )
+	if( m_row_cur[ m_colrec_entry.m_type ] != ColrecEntry::IT_Entry )
+		// all folders must have at least one child:
+		show_row( * m_row_cur.children().begin() );
+
+	if( ! m_textbufferdiary->select_searchstr_previous() )
 	{
-		// ::handle_toolbutton_previous_clicked():
-		if( m_row_cur != ( *m_row_cur.parent() ).children().begin() )
-		{
-			show_row( *( --m_row_cur ), UI_BOTH );
-			m_textbufferdiary->select_searchword_previous();
-		}
+		handle_toolbutton_previous();
+		// put cursor to the end
+		m_textbufferdiary->select_range(	m_textbufferdiary->end(),
+											m_textbufferdiary->end() );
+		m_textbufferdiary->select_searchstr_previous();
 	}
 }
 
@@ -548,18 +694,39 @@ Lifeograph::handle_matchnext( void )
 	if( m_entrycount < 1 )
 		return;
 
-	if( ! m_textbufferdiary->select_searchword_next() )
+	if( m_row_cur[ m_colrec_entry.m_type ] != ColrecEntry::IT_Entry )
+		// all folders must have at least one child:
+		show_row( * m_row_cur.children().begin(), UI_BOTH );
+
+	if( ! m_textbufferdiary->select_searchstr_next() )
 	{
-		// ::handle_toolbutton_next_clicked():
-		Gtk::TreeRow row_next = m_row_cur;
-		if( ++row_next != ( *m_row_cur.parent() ).children().end() )
-		{
-			show_row( row_next, UI_BOTH );
-			m_textbufferdiary->select_searchword_next();
-		}
+		handle_toolbutton_next();
+		m_textbufferdiary->select_searchstr_next();
 	}
 }
 
+void
+Lifeograph::replace_match( void )
+{
+	if( ! m_textbufferdiary->get_has_selection() )
+		handle_matchnext();
+	if( m_textbufferdiary->get_has_selection() )
+	{
+		m_textbufferdiary->erase_selection();
+		m_textbufferdiary->insert_at_cursor( m_entry_replace->get_text() );
+		handle_matchnext();
+	}
+}
+
+void
+Lifeograph::replace_allmatches( void )
+{
+	m_database.replace_filter( m_entry_replace->get_text() );
+	// update current entry:
+	Entry *ptr2entry = static_cast< Entry* >( m_ptr2listitem_cur );
+	m_textbufferdiary->set_richtext( ptr2entry );
+}
+
 void
 Lifeograph::handle_button_backup_clicked( void )
 {
@@ -710,14 +877,15 @@ Lifeograph::handle_calendar_doubleclicked( void )
 	Entry *entry = m_database.create_entry(
 			combine_date( year, month + 1, day ) );
 
+	sync_entry_current();	// synchronize current entry before the list update
 	update_treeview_entries();
 	update_calendar();
 
 	show_listitem( entry, UI_LIST );
 
-	// TODO: give focus to the entry editor
-	//m_textviewdiary->grab_focus();  // does not work because it is probably...
-	// ...not the last signal
+	// this is the only way i could find to give focus to textview
+	Glib::signal_timeout().connect_once(
+			sigc::mem_fun( m_textviewdiary, &Gtk::Widget::grab_focus ), 200 );
 }
 
 void
@@ -727,12 +895,12 @@ Lifeograph::handle_calendar_dayselected( void )
 
 	guint year, month, day;
 	m_calendar->get_date( year, month, day );
-	Entry *entry = m_database.get_entry(
-			combine_date( year, month + 1, day ) );
+	Entry *entry = m_database.get_entry_nextinday(
+			combine_date( year, month + 1, day ),
+			get_entry_current() );
 
 	if( entry )
-		if( entry != m_ptr2listitem_cur )
-			show_listitem( entry, UI_LIST );
+		show_listitem( entry, UI_LIST );
 }
 
 void
@@ -750,9 +918,6 @@ Lifeograph::handle_textview_entry_changed( void )
 
 	m_flag_entrychanged = true;
 
-	// handle emptiness
-	check_db_emptiness();
-
 	if( m_hbox_tagtools->is_visible() )
 		m_hbox_tagtools->hide();
 }
@@ -760,22 +925,42 @@ Lifeograph::handle_textview_entry_changed( void )
 void
 Lifeograph::handle_textview_link_activated( Date date )
 {
-	Entry *entry = m_database.get_entry_create( date );
+	Entryvector *entries = m_database.get_entries( date );
 
-	update_treeview_entries();
-	show_listitem( entry, UI_BOTH );
+	if( entries->size() > 1 )
+	{
+		if( m_menu_link )
+			m_menu_link->items().clear();
+		else
+			m_menu_link = new Gtk::Menu;
+
+		for( Entryiterv iter = entries->begin(); iter != entries->end(); iter++ )
+		{
+			m_menu_link->items().push_back(
+					Gtk::Menu_Helpers::MenuElem(
+							( *iter )->get_title(),
+							sigc::bind(
+									sigc::mem_fun( *this, &Lifeograph::show_listitem ),
+									*iter, UI_BOTH ) ) );
+		}
+
+		m_menu_link->accelerate( *m_window );
+		m_menu_link->popup( 0, gtk_get_current_event_time() );
+	}
+	else
+	if( entries->size() == 1 )
+		show_listitem( entries->at( 0 ), UI_BOTH );
+	else
+	{
+		Entry *entry = m_database.create_entry( date );
+		update_treeview_entries();
+		show_listitem( entry, UI_BOTH );
+
+	}
+	
+	delete entries;
 }
 
-//void
-//Lifeograph::handle_textbuffer_mark_set(	const Gtk::TextIter& iter,
-//										const Glib::RefPtr<Gtk::TextMark>& )
-//{
-//	if( m_textbufferdiary->get_has_selection() )
-//		m_hbox_replace->show();
-//	else
-//		m_hbox_replace->hide();
-//}
-
 LinkStatus
 Lifeograph::handle_textbuffer_link_needs_checking( Date date )
 {
@@ -794,8 +979,9 @@ Lifeograph::handle_textbuffer_title_changed( const Glib::ustring& title )
 {
 	if( m_internaloperation ) return;
 
-	Gtk::TreeRow row = *m_treeview_entries->get_selection()->get_selected();
-	row[ m_colrec_entry.m_info ] = title;
+	m_row_cur[ m_colrec_entry.m_info ] = title;
+
+	update_title();
 }
 
 bool
@@ -836,21 +1022,11 @@ Lifeograph::handle_iconviewtags_buttonpress( GdkEventButton *event )
 	std::list< Gtk::TreeModel::Path > selected_items =
 			m_iconview_tags->get_selected_items();
 
-	bool flag_empty = true;
-
-	for( std::list< Gtk::TreeModel::Path >::iterator iter = selected_items.begin();
-		 iter != selected_items.end();
-		 iter++ )
-	{
-		Gtk::TreeRow row = *m_liststore_tagtable->get_iter( *iter );
-		if( row[ colrec_tag->m_ptr2tag ] != NULL )
-		{
-			flag_empty = false;
-			break;
-		}
-	}
-
-	if( flag_empty )
+	if( selected_items.empty() )
+		return;
+	else
+	if( m_liststore_tagtable->get_iter( selected_items.front() )->get_value(
+				colrec_tag->m_ptr2tag ) == NULL )
 		return;
 
 	if( m_menu_tag )
@@ -873,10 +1049,29 @@ Lifeograph::handle_iconviewtags_buttonpress( GdkEventButton *event )
 	m_menu_tag->popup( event->button, event->time );
 }
 
+void
+Lifeograph::handle_iconviewtags_keyrelease( GdkEventKey *event )
+{
+	if( event->keyval != GDK_Delete )	// the cheapest check first
+		return;
+
+	std::list< Gtk::TreeModel::Path > selected_items =
+			m_iconview_tags->get_selected_items();
+
+	if( selected_items.empty() )
+		return;
+	else
+	if( m_liststore_tagtable->get_iter( selected_items.front() )->get_value(
+				colrec_tag->m_ptr2tag ) == NULL )
+		return;
+
+	handle_removetags();
+}
+
 void
 Lifeograph::handle_tagwidget_activated( void )
 {
-	Entry *ptr2entry = get_treerow_ptr_entry( m_row_cur );
+	Entry *ptr2entry = static_cast< Entry* >( m_ptr2listitem_cur );
 	Tag *tag = Tag::create( m_tagwidget->get_active_text() );
 	if( ptr2entry->get_tags().checkfor_member( tag ) )
 		ptr2entry->get_tags().remove_tag( tag );
@@ -892,71 +1087,6 @@ Lifeograph::handle_tagwidget_activated( void )
 	}
 }
 
-void
-Lifeograph::handle_entry_filter_changed( void )
-{
-	if( m_internaloperation ) return;
-	// current entry must be closed here or else it loses...
-	// ...changes since it was selected:
-	sync_entry_current();
-
-	const std::string filterstring( m_entry_filter->get_text().lowercase() );
-	if( filterstring.size() > 0 )
-	{
-		m_database.set_filter( filterstring );
-		m_textbufferdiary->set_searchword( filterstring );
-
-		update_treeview_entries();
-
-		m_entry_filter->set_tooltip_markup(
-			Glib::ustring::compose(
-				_( "Found in <b>%1</b> of <b>%2</b> entrie(s)" ),
-				m_entrycount, m_database.get_size() ) );
-		m_entry_filter->set_icon_sensitive( Gtk::ENTRY_ICON_SECONDARY, true );
-		m_toolbutton_findprevious->show();
-		m_toolbutton_findnext->show();
-	}
-	else
-	{
-		m_database.remove_filter();
-		m_textbufferdiary->set_search( false );
-
-		update_treeview_entries();
-
-		m_entry_filter->set_tooltip_text( _( "Enter text to be filtered" ) );
-		m_entry_filter->set_icon_sensitive( Gtk::ENTRY_ICON_SECONDARY, false );
-		m_toolbutton_findprevious->hide();
-		m_toolbutton_findnext->hide();
-	}
-
-	update_calendar();
-
-	if( m_entrycount > 0 )
-	{
-		show_row( m_row_cur, UI_BOTH );
-		m_textbufferdiary->select_searchword_next();
-		m_iconview_tags->show();
-	}
-	else
-	{
-		m_internaloperation++;
-
-		m_textbufferdiary->set_richtext( _( "No matches found" ) );
-		m_textviewdiary->set_sensitive( false );
-		m_hbox_tagtools->hide();
-		m_iconview_tags->hide();
-
-		m_internaloperation--;
-	}
-}
-
-void
-Lifeograph::handle_entry_filter_clear( Gtk::EntryIconPosition,
-									   const GdkEventButton* )
-{
-	m_entry_filter->set_text( "" );
-}
-
 void
 Lifeograph::handle_entryliststyle_changed( void )
 {
@@ -1083,12 +1213,12 @@ Lifeograph::show_row( const Gtk::TreeRow& row, Updateindex ui )
 	{
 		m_ptr2listitem_prev = m_ptr2listitem_cur;
 		m_ptr2listitem_cur = row[ m_colrec_entry.m_ptr ];
+		m_row_cur = row;
 	}
-	m_row_cur = row;
 
 	if( row[ m_colrec_entry.m_type ] == ColrecEntry::IT_Entry )
 	{
-		Entry *ptr2entry = get_treerow_ptr_entry( row );
+		Entry *ptr2entry = static_cast< Entry* >( m_ptr2listitem_cur );
 		m_textbufferdiary->set_richtext( ptr2entry );
 
 		if( ui & UI_CAL )
@@ -1167,6 +1297,18 @@ Lifeograph::show_listitem( const void *ptr2listitem, Updateindex ui )
 		show_row( row, ui );
 }
 
+void
+Lifeograph::show_empty( void )
+{
+	m_textbufferdiary->set_richtext( "" );
+	m_textviewdiary->set_sensitive( false );
+	m_window->set_title( _( "Empty Diary" ) );
+	m_action_previous_match->set_sensitive( false );
+	m_action_next_match->set_sensitive( false );
+	m_hbox_tagtools->hide();
+	m_iconview_tags->hide();
+}
+
 void
 Lifeograph::go_back( void )
 {
@@ -1181,7 +1323,7 @@ Lifeograph::sync_entry_current( void )
 		if( m_flag_entrychanged )
 			if( m_row_cur[ m_colrec_entry.m_type ] == ColrecEntry::IT_Entry )
 			{
-				Entry *ptr2entry = get_treerow_ptr_entry( m_row_cur );
+				Entry *ptr2entry = static_cast< Entry* >( m_ptr2listitem_cur );
 				ptr2entry->set_text( m_textbufferdiary->get_text() );
 				m_flag_entrychanged = false;
 			}
@@ -1234,10 +1376,10 @@ Lifeograph::delete_entry( Gtk::TreeRow &row )
 		update_calendar();
 		update_treeview_entries();
 
-		if( check_db_emptiness() > 0 )
-		{
+		if( m_entrycount == 0 )
+			show_empty();
+		else
 			show_row( m_row_cur, UI_BOTH );
-		}
 
 		m_internaloperation--;
 	}
@@ -1260,7 +1402,6 @@ Lifeograph::toggle_entryfavoredness( Gtk::TreeRow &row )
 	}
 }
 
-
 void
 Lifeograph::edit_tag( Tag *tag )
 {
@@ -1294,7 +1435,7 @@ Lifeograph::delete_tag( Tag *tag )
 	if( message.run() != Gtk::RESPONSE_ACCEPT )
 		return;
 
-	Tag::delete_tag( tag );
+	m_database.delete_tag( tag );
 	update_treeview_entries();
 	if( m_entrycount )
 		show_row( m_row_cur, UI_BOTH );
@@ -1314,37 +1455,37 @@ Lifeograph::update_treeview_entries( void )
 	// GROUP BY TAG
 	if( m_database.get_entryliststyle() == LIST_BYTAG )
 	{
-		typedef std::pair< Tag*, Gtk::TreeRow > PairTagRow;
+		typedef std::pair< Tag*, Gtk::TreeRow >	PairTagRow;
+		typedef std::map< Tag*, Gtk::TreeRow >	MapTagRow;
 
-		std::map< Tag*, Gtk::TreeRow >	map_tag_row;
+		MapTagRow						map_tag_row;
 		Gtk::TreeRow					row_untagged;
 		Tag::Tagpool::const_iterator	iter_tag;
 
-		row_untagged = *m_treestore_entries->append();
-		// ptr must be first:
-		row_untagged[ m_colrec_entry.m_ptr ] = &m_address_folderuntagged;
-		row_untagged[ m_colrec_entry.m_type ] = ColrecEntry::IT_Untaggedfolder;
-		row_untagged[ m_colrec_entry.m_id ] = _( "<i>Untagged</i>" );
-		//row_untagged[ m_colrec_entry.m_info ] = // TODO..
-		row_untagged[ m_colrec_entry.m_icon ] = m_pixbuf_folder;
-
-		for(	iter_tag = Tag::get_pool().begin();
-				iter_tag != Tag::get_pool().end();
-				iter_tag++ )
+		// ***********************************************************************
+		// TODO: this part will be removed after a tag editor is introduced
+		if( ! m_database.is_filter_set() )
 		{
-			row_folder = *m_treestore_entries->append();
-			// ptr must be first:
-			row_folder[ m_colrec_entry.m_ptr ] = ( *iter_tag );
-			row_folder[ m_colrec_entry.m_type ] = ColrecEntry::IT_Tagfolder;
-			row_folder[ m_colrec_entry.m_id ] = ( *iter_tag )->get_name();
-			//row_folder[ m_colrec_entry.m_info ] = // TODO..
-			row_folder[ m_colrec_entry.m_icon ] = m_pixbuf_folder;
-			map_tag_row.insert( PairTagRow( *iter_tag, row_folder ) );
+			for(	iter_tag = Tag::get_pool().begin();
+					iter_tag != Tag::get_pool().end();
+					iter_tag++ )
+			{
+				row_folder = *m_treestore_entries->append();
+				// ptr must be first:
+				row_folder[ m_colrec_entry.m_ptr ] = ( *iter_tag );
+				row_folder[ m_colrec_entry.m_type ] = ColrecEntry::IT_Tagfolder;
+				row_folder[ m_colrec_entry.m_id ] =
+						Glib::Markup::escape_text( ( *iter_tag )->get_name() );
+				//row_folder[ m_colrec_entry.m_info ] = // TODO..
+				row_folder[ m_colrec_entry.m_icon ] = m_pixbuf_folder;
+				map_tag_row.insert( PairTagRow( *iter_tag, row_folder ) );
+			}
 		}
+		// ***********************************************************************
 
 		for(	iter_entry = m_database.m_entries.begin();
 				iter_entry != m_database.m_entries.end();
-				iter_entry++ )
+				++iter_entry )
 		{
 			if( ( *iter_entry )->get_filtered_out() == false )
 			{
@@ -1352,10 +1493,22 @@ Lifeograph::update_treeview_entries( void )
 
 				if( ( *iter_entry )->get_tags().empty() )
 				{
+					if( ! row_untagged )
+					{
+						row_untagged = *m_treestore_entries->prepend();
+						// ptr must be first:
+						row_untagged[ m_colrec_entry.m_ptr ] = &m_address_folderuntagged;
+						row_untagged[ m_colrec_entry.m_type ] =
+								ColrecEntry::IT_Untaggedfolder;
+						row_untagged[ m_colrec_entry.m_id ] = _( "<i>Untagged</i>" );
+						//row_untagged[ m_colrec_entry.m_info ] = // TODO..
+						row_untagged[ m_colrec_entry.m_icon ] = m_pixbuf_folder;
+					}
 					row_entry = *m_treestore_entries->append( row_untagged.children() );
 					// ptr must be first:
 					row_entry[ m_colrec_entry.m_ptr ] = ( *iter_entry );
 					row_entry[ m_colrec_entry.m_type ] = ColrecEntry::IT_Entry;
+					row_entry[ m_colrec_entry.m_date_long ] = ( *iter_entry )->get_date();
 					row_entry[ m_colrec_entry.m_id ] = ( *iter_entry )->get_date_str();
 					row_entry[ m_colrec_entry.m_info ] = ( *iter_entry )->get_title();
 					row_entry[ m_colrec_entry.m_icon ] =
@@ -1365,12 +1518,29 @@ Lifeograph::update_treeview_entries( void )
 				}
 				for(	iter_tag = ( *iter_entry )->get_tags().begin();
 						iter_tag != ( *iter_entry )->get_tags().end();
-						iter_tag++ )
+						++iter_tag )
 				{
-					row_entry = *m_treestore_entries->append(
-						map_tag_row.find( *iter_tag )->second.children() );
+					// select row (inserts a new row if it does not exist)
+					MapTagRow::iterator iter_pair = map_tag_row.find( *iter_tag );
+					if( iter_pair != map_tag_row.end() )
+						row_folder = iter_pair->second;
+					else
+					{
+						row_folder = *m_treestore_entries->append();
+						map_tag_row.insert( PairTagRow( *iter_tag, row_folder ) );
+
+						// ptr must be first:
+						row_folder[ m_colrec_entry.m_ptr ] = ( *iter_tag );
+						row_folder[ m_colrec_entry.m_type ] = ColrecEntry::IT_Tagfolder;
+						row_folder[ m_colrec_entry.m_id ] =
+								Glib::Markup::escape_text( ( *iter_tag )->get_name() );
+						//row_folder[ m_colrec_entry.m_info ] = // TODO..
+						row_folder[ m_colrec_entry.m_icon ] = m_pixbuf_folder;
+					}
+					row_entry = *m_treestore_entries->append( row_folder.children() );
 					row_entry[ m_colrec_entry.m_ptr ] = ( *iter_entry );
 					row_entry[ m_colrec_entry.m_type ] = ColrecEntry::IT_Entry;
+					row_entry[ m_colrec_entry.m_date_long ] = ( *iter_entry )->get_date();
 					row_entry[ m_colrec_entry.m_id ] = ( *iter_entry )->get_date_str();
 					row_entry[ m_colrec_entry.m_info ] = ( *iter_entry )->get_title();
 					row_entry[ m_colrec_entry.m_icon ] =
@@ -1455,7 +1625,8 @@ Lifeograph::update_treeview_entries( void )
 				row_folder[ m_colrec_entry.m_ptr ] = ( *iter );
 				row_folder[ m_colrec_entry.m_type ] = ColrecEntry::IT_Chapterfolder;
 				row_folder[ m_colrec_entry.m_date_long ] = ( *iter )->get_date();
-				row_folder[ m_colrec_entry.m_id ] = ( *iter )->get_name();
+				row_folder[ m_colrec_entry.m_id ] =
+								Glib::Markup::escape_text( ( *iter )->get_name() );
 				//row_folder[ m_colrec_entry.m_info ] = // TODO..
 				row_folder[ m_colrec_entry.m_icon ] = m_pixbuf_folder;
 			}
@@ -1509,21 +1680,12 @@ Lifeograph::update_treeview_entries( void )
 void
 Lifeograph::update_toolbar()
 {
-	if( m_row_cur[ m_colrec_entry.m_type ] == ColrecEntry::IT_Entry )
-	{
-		Gtk::TreeRow row_next = m_row_cur;
-		row_next++;
-		m_action_previous_list->set_sensitive(
-				m_row_cur != ( *m_row_cur.parent() ).children().begin() );
-		m_action_next_list->set_sensitive(
-				row_next !=  ( *m_row_cur.parent() ).children().end() );
-	}
-	else
-	{
-		// TODO: show next/previous folder when curent row is a folder
-		m_action_previous_list->set_sensitive( false );
-		m_action_next_list->set_sensitive( false );
-	}
+	bool flag_sensitive =
+			m_row_cur[ m_colrec_entry.m_type ] == ColrecEntry::IT_Entry ?
+					m_entrycount > 1 : m_treestore_entries->children().size() > 1;
+
+	m_action_previous_list->set_sensitive( flag_sensitive );
+	m_action_next_list->set_sensitive( flag_sensitive );
 }
 
 void
@@ -1582,26 +1744,32 @@ Lifeograph::update_tags( void )
 	m_liststore_tagtable->clear();
 	m_liststore_tagcombo->clear();
 
-	Entry *ptr2entry = get_treerow_ptr_entry( m_row_cur );
+	Entry *ptr2entry = static_cast< Entry* >( m_ptr2listitem_cur );
 
 	for(	Tag::Tagpool::const_iterator iter = Tag::get_pool().begin();
 			iter != Tag::get_pool().end();
 			iter++ )
 	{
 		if( ptr2entry->get_tags().checkfor_member( *iter ) )
+		{
 			row = *( m_liststore_tagtable->append() );
+			row[ colrec_tag->m_name ] =
+					Glib::Markup::escape_text( ( *iter )->get_name() );
+		}
 		else
+		{
 			row = *( m_liststore_tagcombo->append() );
+			row[ colrec_tag->m_name ] = ( *iter )->get_name();
+		}
 
 		row[ colrec_tag->m_icon ] = m_pixbuf_tag;
-		row[ colrec_tag->m_name ] = ( *iter )->get_name();
 		row[ colrec_tag->m_ptr2tag ] = ( *iter );
 	}
 
 	if( ptr2entry->get_tags().size() < 1 )
 	{
-		row = ( *( m_liststore_tagtable->append() ) );
-		row[ colrec_tag->m_name ] = _("<i>Untagged Entry</i>");
+		row = *( m_liststore_tagtable->append() );
+		row[ colrec_tag->m_name ] = _( "<i>Untagged Entry</i>" );
 		row[ colrec_tag->m_ptr2tag ] = NULL;
 	}
 
@@ -1761,13 +1929,14 @@ Lifeograph::sort_bydate(	const Gtk::TreeModel::iterator &itr1,
 		itemtype1 == ColrecEntry::IT_Earlier )
 		return 1;
 
-	if( m_database.get_entryliststyle() == LIST_BYTAG )
+	if(	m_database.get_entryliststyle() == LIST_BYTAG &&
+		itemtype1 != ColrecEntry::IT_Entry )
 	{
 		if( ( *itr1 )[ m_colrec_entry.m_id ] > ( *itr2 )[ m_colrec_entry.m_id ] )
-			return -1;
+			return 1;
 		else
 		if( ( *itr1 )[ m_colrec_entry.m_id ] < ( *itr2 )[ m_colrec_entry.m_id ] )
-			return 1;
+			return -1;
 		else
 			return 0;
 	}
@@ -1814,34 +1983,6 @@ Lifeograph::sort_bytitle(	const Gtk::TreeModel::iterator &itr1,
 
 }
 
-bool
-Lifeograph::check_db_emptiness( void )
-{
-	m_internaloperation++;
-
-	// TODO: some repetitions can be got rid of
-	if( m_entrycount == 0 )
-	{
-		m_textbufferdiary->set_richtext( "" );
-		m_textviewdiary->set_sensitive( false );
-		m_window->set_title( _( "Empty Diary" ) );
-		m_action_previous_match->set_sensitive( false );
-		m_action_next_match->set_sensitive( false );
-		m_hbox_tagtools->hide();
-		m_iconview_tags->hide();
-		m_internaloperation--;
-		return false;
-	}
-	else
-	{
-		m_action_previous_match->set_sensitive( true );
-		m_action_next_match->set_sensitive( true );
-		m_iconview_tags->show();
-		m_internaloperation--;
-		return true;
-	}
-}
-
 bool
 Lifeograph::get_entryrow( Gtk::TreeRow &row, const void *ptr )
 {
@@ -1873,6 +2014,18 @@ Lifeograph::get_entryrow( Gtk::TreeRow &row, const void *ptr )
 	return false;
 }
 
+inline Entry*
+Lifeograph::get_entry_current( void )
+{
+	if( m_entrycount < 1 )
+		return NULL;
+	else
+	if( m_row_cur[ m_colrec_entry.m_type ] == ColrecEntry::IT_Entry )
+		return( static_cast< Entry* >( m_ptr2listitem_cur ) );
+	else
+		return NULL;
+}
+
 inline
 Entry*
 Lifeograph::get_treerow_ptr_entry( const Gtk::TreeRow &row )
@@ -1880,8 +2033,7 @@ Lifeograph::get_treerow_ptr_entry( const Gtk::TreeRow &row )
 	return( static_cast< Entry* >( (void*) row[ m_colrec_entry.m_ptr ] ) );
 }
 
-inline
-Tag*
+inline Tag*
 Lifeograph::get_treerow_ptr_tag( const Gtk::TreeRow &row )
 {
 	return( static_cast< Tag* >( (void*) row[ m_colrec_entry.m_ptr ] ) );
