@@ -23,7 +23,7 @@
 #ifndef LIFEOGRAPH_HEADER
 #define LIFEOGRAPH_HEADER
 
-#define LIFEOGRAPH_DEVELOPMENT_VERSION
+//#define LIFEOGRAPH_DEVELOPMENT_VERSION
 
 
 #include <gconfmm.h>
@@ -42,8 +42,8 @@
 namespace LIFEO
 {
 	const char			PROGRAM_NAME[]				= "Lifeograph";
-	const int			PROGRAM_VERSION_INT			= 56;
-	const char			PROGRAM_VERSION_STRING[]	= "0.5.6";
+	const int			PROGRAM_VERSION_INT			= 6030;
+	const char			PROGRAM_VERSION_STRING[]	= "0.6.3";
 
 	const std::string	CONFIG_PATH					= "/apps/lifeograph";
 	const int			BACKUP_INTERVAL				= 30;	// seconds
@@ -52,17 +52,17 @@ namespace LIFEO
 	// LONG AND EPIC TEXT MESSAGES
 	namespace STRING
 	{
-		const char		SLOGAN[]    = N_("Personal, digital diary");
+		const char		SLOGAN[]    = N_(	"Personal, digital diary" );
 
 		const char      WRONG_PASSWORD[]
-                                    = N_("Wrong password. Please retry...");
+                                    = N_(	"Wrong password. Please retry..." );
 
 		const char      ENTER_PASSWORD[]
-                                    = N_("Please enter password for selected diary...");
+                                    = N_(	"Please enter password for selected diary..." );
 
 		const char      ENTER_PASSWORD_TIMEOUT[]
-                                    = N_("Program logged out to protect your privacy. "
-                                         "Please re-enter password...");
+                                    = N_(	"Program logged out to protect your privacy. "
+											"Please re-enter password..." );
 
 		const char		INCOMPATIBLE_DB[]
 									= N_(	"Selected diary is not compatible with "
@@ -74,29 +74,27 @@ namespace LIFEO
 											"Lifeograph diary. "
 											"Please select another file..." );
 
-		const char      SELECT_DB[] = N_("Please select a diary...");
-
 		const char      FAILED_TO_OPEN_DB[]
-                                    = N_("Failed to open diary. "
-                                         "Please select anoter file...");
+                                    = N_(	"Failed to open diary. "
+											"Please select anoter file..." );
 
 		const char      DB_IS_NOT_ENCRYPTED[]
-                                    = N_("Unencrypted diary. Press OK to open");
+                                    = N_(	"Unencrypted diary. Press OK to open" );
 
 		const char      SALUTATION[]    /*  TRANSLATORS: this is the message
                                             shown to the fist time users */
-                                    = N_("Knowing others is intelligence;\n"
-                                         "knowing yourself is true wisdom.\n"
-                                         "Mastering others is strength;\n"
-                                         "mastering yourself is true power.\n\n"
-                                         "<i>--Lao Tzu</i>");
+                                    = N_(	"Knowing others is intelligence;\n"
+											"knowing yourself is true wisdom.\n"
+											"Mastering others is strength;\n"
+											"mastering yourself is true power.\n\n"
+											"<i>--Lao Tzu</i>" );
 
 		const char      CANNOT_WRITE[]
-                                    = N_("Changes could not be written to diary!");
+                                    = N_(	"Changes could not be written to diary!" );
 
 		const char      CANNOT_WRITE_SUB[]
-                                    = N_("Check if you have write permissions on the "
-                                         "file and containing folder");
+                                    = N_(	"Check if you have write permissions on the "
+											"file and containing folder" );
 
 	}
 
@@ -166,6 +164,7 @@ class Lifeograph
 		void				show_row( const Gtk::TreeRow&, Updateindex = UI_NONE );
 		void				show_listitem( const void*, Updateindex = UI_NONE );
 		void				show_today( void );
+		void				show_empty( void );
 		void				go_back( void );
 		void				sync_entry_current( void );	// synchronize with db
 		void				present_entryrow( const Gtk::TreeRow& );
@@ -224,15 +223,13 @@ class Lifeograph
 
 		void				handle_textview_entry_changed( void );
 		void				handle_textview_link_activated( Date );
-//		void				handle_textbuffer_mark_set(
-//									const Gtk::TextIter&,
-//									const Glib::RefPtr<Gtk::TextMark>& );
 		LinkStatus			handle_textbuffer_link_needs_checking( Date );
 		void				handle_textbuffer_title_changed( const Glib::ustring& );
 
 		bool				handle_hboxtagtools_enternotify( GdkEventCrossing* );
 		void				handle_removetags( void );
 		void				handle_iconviewtags_buttonpress( GdkEventButton* );
+		void				handle_iconviewtags_keyrelease( GdkEventKey* );
 		void				handle_tagwidget_activated( void );
 
 		void				handle_calendar_doubleclicked( void );
@@ -244,6 +241,8 @@ class Lifeograph
 									Gtk::EntryIconPosition, const GdkEventButton* );
 		void				handle_matchprevious( void );
 		void				handle_matchnext( void );
+		void				replace_match( void );
+		void				replace_allmatches( void );
 
 		void				handle_undo( void );
 		void				handle_redo( void );
@@ -285,15 +284,16 @@ class Lifeograph
 		TextviewDiary				*m_textviewdiary;
 		Gtk::Calendar				*m_calendar;
 		EntryIdletext				*m_entry_filter;
-//		Gtk::Entry					*m_entry_replace;
-//		Gtk::Button					*m_button_replace;
-//		Gtk::Button					*m_button_replaceall;
+		Gtk::Entry					*m_entry_replace;
+		Gtk::Button					*m_button_replace;
+		Gtk::Button					*m_button_replaceall;
 		Gtk::Menu					*m_menu_treeviewentries;
 		Gtk::Menu					*m_menu_tag;
+		Gtk::Menu					*m_menu_link;
 
 		Gtk::HPaned					*m_hpaned_main;
 		Gtk::HBox					*m_hbox_tagtools;
-//		Gtk::HBox					*m_hbox_replace;
+		Gtk::HBox					*m_hbox_replace;
 
 		Gtk::ToolButton				*m_toolbutton_previous;
 		Gtk::ToolButton				*m_toolbutton_next;
@@ -343,6 +343,7 @@ class Lifeograph
 		// VARIABLES
 		const std::string			m_programpath;
 		LoginStatus					m_loginstatus;
+		bool						m_flag_firsttimeuser;
 		int							m_option_idletime_max;
 		bool						m_option_autologout;
 		std::string					m_option_bg_path;
@@ -358,8 +359,8 @@ class Lifeograph
 		int							m_address_folderearlier;	// address for void* in row
 
 		// FUNCTIONS
-		bool						check_db_emptiness( void );
 		bool						get_entryrow( Gtk::TreeRow&, const void* );
+		Entry *						get_entry_current( void );
 		Entry *						get_treerow_ptr_entry( const Gtk::TreeRow& );
 		Tag *						get_treerow_ptr_tag( const Gtk::TreeRow& );
 
