@@ -51,23 +51,21 @@ void
 Lifeograph::draw_welcomescreen( void )
 {
 	Gtk::Alignment *alignment = Gtk::manage( new Gtk::Alignment( 0.5, 0.5, 0.1, 0 ) );
-
-	alignment->set_padding( 100, 100, 100, 100 );
-	m_window->add( *alignment );
-
 	Gtk::Label *label_salutation = Gtk::manage(
 		new Gtk::Label( _( STRING::SALUTATION ), 0.5, 0.5 ) );
-	label_salutation->set_use_markup( true );
-
 	Gtk::Button *button_start = Gtk::manage( new Gtk::Button( _("_Continue"), true ) );
 	Gtk::Image *icon_forward = Gtk::manage(
 		new Gtk::Image( Gtk::Stock::GO_FORWARD, Gtk::ICON_SIZE_MENU ) );
+	Gtk::VBox *vbox = Gtk::manage( new Gtk::VBox );
+
+	alignment->set_padding( 120, 120, 150, 150 );
+	m_window->add( *alignment );
+
+	label_salutation->set_use_markup( true );
+
 	button_start->set_image( *icon_forward );
 	button_start->set_image_position( Gtk::POS_RIGHT );
-	button_start->signal_clicked().connect(
-		sigc::mem_fun( *this, &Lifeograph::draw_loginscreen ) );
 
-	Gtk::VBox *vbox = Gtk::manage( new Gtk::VBox );
 	vbox->set_spacing( 50 );
 
 	vbox->pack_start( *label_salutation );
@@ -75,6 +73,13 @@ Lifeograph::draw_welcomescreen( void )
 
 	alignment->add( *vbox );
 
+	button_start->signal_clicked().connect(
+		sigc::mem_fun( *this, &Lifeograph::draw_loginscreen ) );
+
+	if( bool( m_pixbuf_background ) )
+		m_connection_event_expose = m_window->signal_expose_event().connect(
+				sigc::mem_fun( *this, &Lifeograph::handle_event_expose ) );
+
 	m_window->show_all();
 }
 
@@ -171,7 +176,7 @@ Lifeograph::draw_loginscreen( void )
 	button_about->signal_clicked().connect(
 			sigc::mem_fun( *this, &Lifeograph::handle_button_about_clicked ) );
 
-	if( bool( m_pixbuf_background ) )
+	if( bool( m_pixbuf_background ) && m_connection_event_expose.empty() )
 		m_connection_event_expose = m_window->signal_expose_event().connect(
 				sigc::mem_fun( *this, &Lifeograph::handle_event_expose ) );
 
@@ -184,7 +189,7 @@ Lifeograph::draw_loginscreen( void )
 	{
 		m_entry_password->set_sensitive( false );
 		m_button_opendiary->set_sensitive( false );
-		m_label_startupmessage->set_label( _( STRING::SELECT_DB ) );
+		m_label_startupmessage->set_label( _( "Welcome new user!" ) );
 	}
 
 	m_window->show_all();
@@ -198,7 +203,8 @@ Lifeograph::draw_editscreen( void )
 {
 	m_connection_event_expose.disconnect();
 	// Filebutton has to be unparented to be used again:
-	m_table_login->remove( *m_filebuttonrecent );
+	if( m_filebuttonrecent->get_parent() )
+		m_table_login->remove( *m_filebuttonrecent );
 	m_window->remove();
 
 	Gtk::VBox				*vbox, *vbox_entry, *vbox_navigation;
@@ -258,8 +264,6 @@ Lifeograph::draw_editscreen( void )
 			new Gtk::CheckMenuItem( _( "Log out Automatically on _Idle" ),
 									true ) );
 
-		if( m_textbufferdiary == NULL )
-			m_textbufferdiary = new TextbufferDiary( &m_undomanager, &m_database );
 		m_textviewdiary = Gtk::manage( new TextviewDiary( m_textbufferdiary ) );
 
 		scrolledwindow1 = Gtk::manage( new Gtk::ScrolledWindow );
@@ -268,11 +272,11 @@ Lifeograph::draw_editscreen( void )
 
 		m_hpaned_main = Gtk::manage( new Gtk::HPaned );
 		m_hbox_tagtools = Gtk::manage( new Gtk::HBox );
-//		m_hbox_replace = Gtk::manage( new Gtk::HBox );
+		m_hbox_replace = Gtk::manage( new Gtk::HBox );
 		m_entry_filter = Gtk::manage( new EntryIdletext( _( "Filter" ) ) );
-//		m_entry_replace = Gtk::manage( new Gtk::Entry );
-//		m_button_replace = Gtk::manage( new Gtk::Button( _( "Replace" ) ) );
-//		m_button_replaceall = Gtk::manage( new Gtk::Button( _( "Replace All" ) ) );
+		m_entry_replace = Gtk::manage( new Gtk::Entry );
+		m_button_replace = Gtk::manage( new Gtk::Button( _( "Replace" ) ) );
+		m_button_replaceall = Gtk::manage( new Gtk::Button( _( "Replace All" ) ) );
 		m_calendar = Gtk::manage( new Gtk::Calendar );
 		m_treeview_entries = Gtk::manage( new Gtk::TreeView );
 		m_combobox_liststyle = Gtk::manage( new Gtk::ComboBoxText );
@@ -331,13 +335,17 @@ Lifeograph::draw_editscreen( void )
 	scrolledwindow1->set_size_request( 300, -1 );
 	scrolledwindow1->add( *m_textviewdiary );
 
+	// CALENDAR
+	m_calendar->set_tooltip_text( _( "Double click a day to add a new entry" ) );
+
+	// PACKING
 	vbox_navigation->pack_start( *scrolledwindow3 );
 	vbox_navigation->pack_start( *m_combobox_liststyle, Gtk::PACK_SHRINK );
 	vbox_navigation->pack_start( *m_calendar, Gtk::PACK_SHRINK );
 
 	m_hbox_tagtools->pack_start( *m_tagwidget );
 
-//	vbox_entry->pack_start( *m_hbox_replace, Gtk::PACK_SHRINK );
+	vbox_entry->pack_start( *m_hbox_replace, Gtk::PACK_SHRINK );
 	vbox_entry->pack_start( *scrolledwindow1 );
 	vbox_entry->pack_start( *m_hbox_tagtools, Gtk::PACK_SHRINK );
 	vbox_entry->pack_start( *scrolledwindow2, Gtk::PACK_SHRINK );
@@ -413,7 +421,7 @@ Lifeograph::draw_editscreen( void )
 	// initially sort entries by date
 	m_treestore_entries->set_sort_column( m_colrec_entry.m_id, Gtk::SORT_ASCENDING );
 
-	// TAG TABLE
+	// TAG VIEW
 	m_iconview_tags->set_pixbuf_column( colrec_tag->m_icon );
 	m_iconview_tags->set_markup_column( colrec_tag->m_name );
 	m_iconview_tags->set_model( m_liststore_tagtable );
@@ -427,12 +435,15 @@ Lifeograph::draw_editscreen( void )
 	m_tagwidget->set_text_column( colrec_tag->m_name );
 
 	// REPLACE BAR
-//	m_hbox_replace->set_spacing( 5 );
-//	m_hbox_replace->set_border_width( 5 );
-//	m_hbox_replace->pack_start( *label_replace, Gtk::PACK_SHRINK );
-//	m_hbox_replace->pack_start( *m_entry_replace );
-//	m_hbox_replace->pack_start( *m_button_replace );
-//	m_hbox_replace->pack_start( *m_button_replaceall );
+	m_entry_replace->set_tooltip_text( _( "Enter text to replace matches with" ) );
+	m_button_replace->set_tooltip_text( _( "Replace selected (if any) or next match" ) );
+	m_button_replaceall->set_tooltip_text(
+			_( "Replace all matches throughout entire diary" ) );
+	m_hbox_replace->set_spacing( 5 );
+	m_hbox_replace->set_border_width( 5 );
+	m_hbox_replace->pack_start( *m_entry_replace );
+	m_hbox_replace->pack_start( *m_button_replace, Gtk::PACK_SHRINK );
+	m_hbox_replace->pack_start( *m_button_replaceall, Gtk::PACK_SHRINK );
 
 	m_window->add( *vbox );
 
@@ -561,16 +572,7 @@ Lifeograph::draw_editscreen( void )
 
 	m_toolbutton_logout->signal_clicked().connect(
 		sigc::bind( sigc::mem_fun( *this, &Lifeograph::logout ), true ) );
-	m_textbufferdiary->signal_changed().connect(
-		sigc::mem_fun( *this, &Lifeograph::handle_textview_entry_changed ) );
-//	m_textbufferdiary->signal_mark_set().connect(
-//		sigc::mem_fun( *this, &Lifeograph::handle_textbuffer_mark_set ) );
-	m_textbufferdiary->signal_link_needs_checking().connect(
-		sigc::mem_fun( *this, &Lifeograph::handle_textbuffer_link_needs_checking ) );
-	m_textbufferdiary->signal_title_changed().connect(
-		sigc::mem_fun( *this, &Lifeograph::handle_textbuffer_title_changed ) );
-	LinkEntry::signal_activated().connect(
-		sigc::mem_fun( *this, &Lifeograph::handle_textview_link_activated ) );
+
 	m_treeview_entries->get_selection()->signal_changed().connect(
 		sigc::mem_fun( *this, &Lifeograph::handle_treeview_entries_selectionchanged ) );
 	m_treeview_entries->signal_button_press_event().connect_notify(
@@ -585,6 +587,8 @@ Lifeograph::draw_editscreen( void )
 		sigc::mem_fun( *this, &Lifeograph::handle_hboxtagtools_enternotify ) );
 	m_iconview_tags->signal_button_press_event().connect_notify(
 		sigc::mem_fun( *this, &Lifeograph::handle_iconviewtags_buttonpress ) );
+	m_iconview_tags->signal_key_release_event().connect_notify(
+		sigc::mem_fun( *this, &Lifeograph::handle_iconviewtags_keyrelease ) );
 	m_tagwidget->signal_activate().connect(
 		sigc::mem_fun( *this, &Lifeograph::handle_tagwidget_activated ) );
 
@@ -601,6 +605,11 @@ Lifeograph::draw_editscreen( void )
 		sigc::mem_fun( *this, &Lifeograph::handle_matchnext ) );
 	m_entry_filter->signal_icon_release().connect(
 		sigc::mem_fun( *this, &Lifeograph::handle_entry_filter_clear ) );
+	m_button_replace->signal_clicked().connect(
+		sigc::mem_fun( *this, &Lifeograph::replace_match ) );
+	m_button_replaceall->signal_clicked().connect(
+		sigc::mem_fun( *this, &Lifeograph::replace_allmatches ) );
+
 
 	if( m_database.is_encrypted() && m_option_autologout )
 		m_connection_event = m_window->signal_event().connect(
@@ -611,7 +620,7 @@ Lifeograph::draw_editscreen( void )
 			BACKUP_INTERVAL );
 
 	m_window->show_all();
-//	m_hbox_replace->hide();
+	m_hbox_replace->hide();
 	m_toolbutton_findprevious->hide();
 	m_toolbutton_findnext->hide();
 }
