@@ -113,8 +113,10 @@ LinkUri::go( void ) const
 TextbufferDiary::TextbufferDiary( UndoManager *undomanager, Database *ptr2database  )
 	:	Gtk::TextBuffer(), m_title( "" ), m_option_search( false ),
 	m_flag_settextoperation( false ), m_flag_ongoingoperation( false ),
+	m_flag_parsing( false ),
 	m_ptr2undomanager( undomanager ),
-	m_ptr2database( ptr2database ), m_ptr2textview( NULL ), m_ptr2spellobj( NULL )
+	m_ptr2database( ptr2database ), m_ptr2entry( NULL ),
+	m_ptr2textview( NULL ), m_ptr2spellobj( NULL )
 {
 	LinkUri::m_ptr2buffer = this;
 	UndoEdit::m_ptr2buffer = this;
@@ -163,8 +165,9 @@ TextbufferDiary::TextbufferDiary( UndoManager *undomanager, Database *ptr2databa
 void
 TextbufferDiary::reset( void )
 {
-	set_searchword( "" );
+	set_searchstr( "" );
 	m_title.clear();
+	set_spellcheck( false );
 }
 
 Glib::ustring
@@ -180,51 +183,71 @@ TextbufferDiary::set_search( bool option_search )
 }
 
 void
-TextbufferDiary::set_searchword( const Glib::ustring &searchword )
+TextbufferDiary::set_searchstr( const Glib::ustring &searchstr )
 {
-	m_searchword = searchword;
-	// disable search if searchword is empty:
-	m_option_search = static_cast< bool >( searchword.size() );
+	m_searchstr = searchstr;
+	// disable search if searchstr is empty:
+	m_option_search = static_cast< bool >( searchstr.size() );
 }
 
 bool
-TextbufferDiary::select_searchword_previous( void )
+TextbufferDiary::select_searchstr_previous( void )
 {
 	if( ! m_option_search )
 		return false;
 
-	Gtk::TextIter iter_start = end(), iter_end = end();
-	if( get_has_selection() )
-	{
-		get_selection_bounds( iter_start, iter_end );
-	}
+	Gtk::TextIter iter_start, iter_end, iter_bound_start, iter_bound_end;
+	get_selection_bounds( iter_bound_start, iter_bound_end );
 
-	if( ! iter_start.backward_to_tag_toggle( m_tag_match ) )
+	if( iter_bound_start.is_start() )
 		return false;
-	iter_end = iter_start;
-	iter_start.backward_to_tag_toggle( m_tag_match );
+
+	iter_bound_end = iter_bound_start;
+	
+	if( ! iter_bound_start.backward_to_tag_toggle( m_tag_match ) )
+		return false;
+	iter_bound_start.backward_to_tag_toggle( m_tag_match );
+	iter_bound_start.backward_to_tag_toggle( m_tag_match );
+
+	if( ! iter_bound_end.backward_search(	m_searchstr,
+											Gtk::TextSearchFlags( 0 ),
+											iter_start,
+											iter_end,
+											iter_bound_start ) )
+		return false;
+
 	select_range( iter_start, iter_end );
 	m_ptr2textview->scroll_to( iter_start );
 	return true;
 }
 
 bool
-TextbufferDiary::select_searchword_next( void )
+TextbufferDiary::select_searchstr_next( void )
 {
 	if( ! m_option_search )
 		return false;
 
-	Gtk::TextIter iter_start = begin(), iter_end = begin();
-	if( get_has_selection() )
-	{
-		get_selection_bounds( iter_start, iter_end );
-	}
+	Gtk::TextIter iter_start, iter_end, iter_bound_start, iter_bound_end;
+	get_selection_bounds( iter_bound_start, iter_bound_end );
+
+	if( iter_bound_end.is_end() )
+		return false;
+
+	iter_bound_start = iter_bound_end;
+
+	if( ! iter_bound_end.forward_to_tag_toggle( m_tag_match ) )
+		return false;
+	iter_bound_end.forward_to_tag_toggle( m_tag_match );
+	if( iter_bound_end.has_tag( m_tag_match ) )
+		iter_bound_end.forward_to_tag_toggle( m_tag_match );
+
+	if( ! iter_bound_start.forward_search(	m_searchstr,
+													Gtk::TextSearchFlags( 0 ),
+													iter_start,
+													iter_end,
+													iter_bound_end ) )
+		return false;
 
-	if( ! iter_end.has_tag( m_tag_match ) )
-		if( ! iter_end.forward_to_tag_toggle( m_tag_match ) )
-			return false;
-	iter_start = iter_end;
-	iter_end.forward_to_tag_toggle( m_tag_match );
 	select_range( iter_start, iter_end );
 	m_ptr2textview->scroll_to( iter_start );
 	return true;
@@ -233,6 +256,7 @@ TextbufferDiary::select_searchword_next( void )
 void
 TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 {
+	m_flag_parsing = true;
 	// BEGINNING & END
 	Gtk::TextIter	iter_start = get_iter_at_offset( pos_start );
 	Gtk::TextIter	const iter_end = get_iter_at_offset( pos_end );
@@ -248,16 +272,16 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 	gunichar		char_current;
 	Glib::ustring	word_last = "";
 	unsigned int	int_last = 0;
-	Date			date_link;
+	Date			date_link = 0;
 
 	// SEARCHING
 	gunichar		char_search = 0;
 	Gtk::TextIter	iter_search = iter_start;
 	Gtk::TextIter	iter_word = iter_start;
 	size_t			pos_search = 0;
-	const size_t	pos_search_end = m_searchword.size();
+	const size_t	pos_search_end = m_searchstr.size();
 
-	for( ; ; iter_current++ )
+	for( ; ; ++iter_current )
 	{
 		char_current = iter_current.get_char();
 		// SEARCH WORD MATCHING
@@ -269,7 +293,7 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 				apply_tag( m_tag_match, iter_search, iter_current );
 				pos_search = 0;
 			}
-			char_search = m_searchword[ pos_search ];
+			char_search = m_searchstr[ pos_search ];
 			if( char_search == Glib::Unicode::tolower( char_current ) )
 			{
 				if( pos_search == 0 )
@@ -292,7 +316,10 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 				{
 					case LF_FIRSTNEWLINE:
 						apply_tag( m_tag_heading, iter_start, iter_current );
-						m_title = get_slice( iter_start, iter_current );
+						if( char_current == 0 && iter_current.get_offset() == 0 )
+							m_title = _( "<empty entry>" );
+						else
+							m_title = get_slice( iter_start, iter_current );
 						if( ! m_flag_settextoperation )
 							m_signal_title_changed.emit( m_title );
 						lookingfor = LF_SPACE;
@@ -357,24 +384,27 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 			break;
 			case '0': case '1': case '2': case '3': case '4':
 			case '5': case '6': case '7': case '8': case '9':
+				if( char_last == '1' )
+				{
+					int_last *= 10;
+					int_last += ( char_current - '0' );
+				}
+				else
+					int_last = ( char_current - '0' );
 				switch ( lookingfor )
 				{
 					case LF_NOTHING:
 					case LF_SPACE:
-						int_last = ( char_current - '0' ) * 1000;
 						lookingfor = LF_DIGITYEAR2;
 						iter_start = iter_current;
 						break;
 					case LF_DIGITYEAR2:
-						int_last += ( char_current - '0' ) * 100;
-						lookingfor = LF_DIGITYEAR3;
-						break;
 					case LF_DIGITYEAR3:
-						int_last += ( char_current - '0' ) * 10;
-						lookingfor = LF_DIGITYEAR4;
+					case LF_DIGITMONTH1:
+					case LF_DIGITDAY1:
+						lookingfor = LookingFor( lookingfor + 1 );
 						break;
 					case LF_DIGITYEAR4:
-						int_last += ( char_current - '0' );
 						if(	int_last > YEAR_MAX || int_last < YEAR_MIN )
 							lookingfor = LF_NOTHING;
 						else
@@ -383,12 +413,7 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 							lookingfor = LF_DOTYM;
 						}
 						break;
-					case LF_DIGITMONTH1:
-						int_last = ( char_current - '0' ) * 10;
-						lookingfor = LF_DIGITMONTH2;
-						break;
 					case LF_DIGITMONTH2:
-						int_last += ( char_current - '0' );
 						if(	int_last > 12 || int_last < 1 )
 							lookingfor = LF_NOTHING;
 						else
@@ -397,12 +422,8 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 							lookingfor = lookingfor2;
 						}
 						break;
-					case LF_DIGITDAY1:
-						date_link |= ( char_current - '0' ) * 10;
-						lookingfor = LF_DIGITDAY2;
-						break;
 					case LF_DIGITDAY2:
-						date_link += ( char_current - '0' );
+						date_link |= int_last;
 						add_link_date(	date_link,
 										lookingfor,
 										iter_start,
@@ -547,6 +568,8 @@ TextbufferDiary::parse( unsigned int pos_start, unsigned int pos_end )
 		if( iter_current == iter_end )
 			break;
     }
+
+	m_flag_parsing = false;
 }
 
 // HELPER FUNCTIONS FOR PARSING
@@ -645,7 +668,7 @@ TextbufferDiary::add_link(	const std::string& url,
 
 // TODO: this function might be combined with add_link()
 inline void
-TextbufferDiary::add_link_date(	const Date &date,
+TextbufferDiary::add_link_date(	const Date date,
 								LookingFor& lookingfor,
 								const Gtk::TextIter& iter_start,
 								const Gtk::TextIter& iter_end )
@@ -680,17 +703,27 @@ TextbufferDiary::set_richtext( const Glib::ustring &text )
 	m_title = _( "<empty entry>" );
 	m_ptr2undomanager->freeze();
 	Gtk::TextBuffer::set_text( text );
+	place_cursor( begin() );
 	m_ptr2undomanager->thaw();
+	m_ptr2entry = NULL;
 	m_flag_settextoperation = false;
 }
 
 void
-TextbufferDiary::set_richtext( const Entry *entry )
+TextbufferDiary::set_richtext( Entry *entry )
 {
 	set_richtext( entry->get_text() );
+	m_ptr2entry = entry;
 	// TODO: evaluate per entry spellcheck options here
 }
 
+void
+TextbufferDiary::reparse( void )
+{
+	place_cursor( begin() );
+	parse( 0, get_char_count() );
+}
+
 void
 TextbufferDiary::on_insert(	const Gtk::TextIter& iterator,
 							const Glib::ustring& text,
@@ -775,6 +808,18 @@ TextbufferDiary::on_erase(	const Gtk::TextIter& iter_start,
 	}
 }
 
+void
+TextbufferDiary::on_remove_tag(	const Glib::RefPtr< TextBuffer::Tag >& tag,
+								const Gtk::TextIter& iter_begin,
+								const Gtk::TextIter& iter_end )
+{
+	// do not remove gtkspell tags while parsing:
+	if( m_flag_parsing && tag->property_name() == "gtkspell-misspelled" )
+		return;
+	else
+		Gtk::TextBuffer::on_remove_tag( tag, iter_begin, iter_end );
+}
+
 const Link*
 TextbufferDiary::get_link( int offset ) const
 {
@@ -868,24 +913,74 @@ void
 TextbufferDiary::toggle_format(	Glib::RefPtr< Tag > tag, const Glib::ustring &markup )
 {
 	Gtk::TextIter iter_start, iter_end;
-	if( ! get_has_selection() )
+	if( get_has_selection() )
+	{
+		get_selection_bounds( iter_start, iter_end );
+		iter_end--;
+		int pos_start = -1, pos_end = -1;
+
+		for( ; ; iter_start++ )
+		{
+			if( iter_start.has_tag( tag ) )
+				return;
+			switch( iter_start.get_char() )
+			{
+				// do nothing if selection spreads over more than one line:
+				case '\n':
+					return;
+				case ' ':
+				case '\t':
+					break;
+				case '*':
+				case '_':
+				case '#':
+				case '=':
+					if( iter_start.get_char() == markup[ 0 ] )
+						break;
+				default:
+					if( pos_start < 0 )
+						pos_start = iter_start.get_offset();
+					pos_end = iter_start.get_offset();
+					break;
+			}
+			if( iter_start == iter_end )
+				break;
+		}
+		// add markup chars to the beginning and end:
+		if( pos_start >= 0 )
+		{
+			pos_end += 2;
+			insert( get_iter_at_offset( pos_start ), markup );
+			insert( get_iter_at_offset( pos_end ), markup );
+			place_cursor( get_iter_at_offset( pos_end ) );
+		}
+	}
+	else	// no selection case
 	{
 		Glib::RefPtr< Gtk::TextMark > mark = get_insert();
 		iter_start = mark->get_iter();
-		if(	! Glib::Unicode::isalpha( iter_start.get_char() ) &&
-			! Glib::Unicode::isdigit( iter_start.get_char() ) )
-			return;
-		else
+		if( Glib::Unicode::isspace( iter_start.get_char() ) ||
+			iter_start.has_tag( TextbufferDiary::m_tag_markup ) )
+		{
+			if( ! iter_start.starts_line() )
+				iter_start--;
+			else
+				return;
+		}
 		if( iter_start.has_tag( tag ) )
 		{
 			m_flag_ongoingoperation = true;
 
 			// find start and end points of formatting:
+			if( iter_start.starts_word() )
+				iter_start++;	// necessary when cursor is between a space char
+								// and non-space char
 			iter_start.backward_to_tag_toggle( tag );
 			backspace( iter_start );
 
 			iter_end = mark->get_iter();
-			iter_end.forward_to_tag_toggle( tag );
+			if( iter_end.has_tag( tag ) )
+				iter_end.forward_to_tag_toggle( TextbufferDiary::m_tag_markup );
 
 			m_flag_ongoingoperation = false;
 
@@ -895,45 +990,26 @@ TextbufferDiary::toggle_format(	Glib::RefPtr< Tag > tag, const Glib::ustring &ma
 		if( iter_start.get_tags().empty() )		// nested tags are not supported atm
 		{
 			// find word boundaries:
-			if( ! iter_start.starts_word() && ! iter_start.starts_line() )
+			if( !( iter_start.starts_word() || iter_start.starts_line() ) )
 				iter_start.backward_word_start();
 			insert( iter_start, markup );
 
 			iter_end = mark->get_iter();
-			if( ! iter_end.ends_word() && ! iter_end.ends_line() )
+			if( !( iter_end.ends_word() || iter_end.ends_line() ) )
+			{
 				iter_end.forward_word_end();
-			insert( iter_end, markup );
-		}
-	}
-	else
-	{
-		get_selection_bounds( iter_start, iter_end );
-		Glib::ustring text = get_slice( iter_start, iter_end );
-		iter_end--;
-
-		// do nothing if selection spreads over more than one line:
-		if( text.find( '\n' ) != Glib::ustring::npos )
-			return;
-
-		bool flag_omit_start = iter_start.has_tag( tag );
-		bool flag_omit_end = iter_end.has_tag( tag );
-
-		// erase markup chars:
-		for( Glib::ustring::size_type i = 0; i < text.size(); )
-		{
-			if( text[ i ] == markup[ 0 ] )
-				text.erase( i, 1 );
+				insert( iter_end, markup );
+			}
 			else
-				i++;
+			{
+				int offset = iter_end.get_offset();
+				insert( iter_end, markup );
+				place_cursor( get_iter_at_offset( offset ) );
+
+			}
 		}
-		// add markup chars to the beginning and end:
-		if( ! flag_omit_start )
-			text.insert( 0, markup );
-		if( ! flag_omit_end )
-			text.insert( text.size(), markup );
-		erase_selection();
-		insert_at_cursor( text );
 	}
+
 }
 
 void
@@ -965,7 +1041,8 @@ TextbufferDiary::set_spellcheck( bool option_spell )
 {
 	if( option_spell )
 	{
-		m_ptr2spellobj = gtkspell_new_attach( m_ptr2textview->gobj(), NULL, NULL );
+		if( ! m_ptr2spellobj )
+			m_ptr2spellobj = gtkspell_new_attach( m_ptr2textview->gobj(), NULL, NULL );
 	}
 	else
 	if( m_ptr2spellobj )
@@ -974,6 +1051,7 @@ TextbufferDiary::set_spellcheck( bool option_spell )
 		m_ptr2spellobj = NULL;
 	}
 
+	// FIXME: here is not the most right place to do this!
 	m_ptr2database->set_spellcheck( option_spell );
 }
 
@@ -1037,21 +1115,17 @@ TextviewDiary::update_link( void )
     Gdk::ModifierType	modifiers;
 
     Gtk::Widget::get_window()->get_pointer( pointer_x, pointer_y, modifiers );
+	window_to_buffer_coords(	Gtk::TEXT_WINDOW_WIDGET,
+								pointer_x, pointer_y,
+								buffer_x, buffer_y );
+	get_iter_at_location( iter, buffer_x, buffer_y );
+	m_link_hovered = m_ptr2buffer->get_link( iter.get_offset() );
 
-	if( !( modifiers & Gdk::CONTROL_MASK ) )
+	if( m_link_hovered != NULL )
 	{
-		window_to_buffer_coords(	Gtk::TEXT_WINDOW_WIDGET,
-									pointer_x, pointer_y,
-									buffer_x, buffer_y );
-
-		get_iter_at_location( iter, buffer_x, buffer_y );
-		m_link_hovered = m_ptr2buffer->get_link( iter.get_offset() );
-
-		if( m_link_hovered != NULL )
+		if( !( modifiers & Gdk::CONTROL_MASK ) )
 			ptr2cursor = &m_cursor_hand;
 	}
-	else
-		m_link_hovered = NULL;
 
 	if( ptr2cursor != m_ptr2cursor_last )
 	{
@@ -1071,21 +1145,22 @@ bool
 TextviewDiary::on_button_release_event( GdkEventButton *event )
 {
 	if( m_link_hovered != NULL )
-		m_link_hovered->go();
+		if( ( event->state & Gdk::CONTROL_MASK ) != Gdk::CONTROL_MASK )
+			m_link_hovered->go();
 
 	return Gtk::TextView::on_button_release_event( event );
 }
 
 bool
-TextviewDiary::on_key_press_event( GdkEventKey* event )
+TextviewDiary::on_key_press_event( GdkEventKey *event )
 {
-	if( event->keyval == GDK_Control_L || event->keyval == GDK_Control_R  )
+	if( event->keyval == GDK_Control_L || event->keyval == GDK_Control_R )
 	{
 		if( m_link_hovered )
 			update_link();
 	}
-	else
-	if( event->state == Gdk::CONTROL_MASK )
+
+	if( ( event->state & Gdk::CONTROL_MASK ) == Gdk::CONTROL_MASK )
 	{
 		switch( event->keyval )
 		{
@@ -1108,10 +1183,11 @@ TextviewDiary::on_key_press_event( GdkEventKey* event )
 }
 
 bool
-TextviewDiary::on_key_release_event( GdkEventKey* event )
+TextviewDiary::on_key_release_event( GdkEventKey *event )
 {
 	if( event->keyval == GDK_Control_L || event->keyval == GDK_Control_R )
-		update_link();
+		if( m_link_hovered )
+			update_link();
 		
 	return Gtk::TextView::on_key_release_event( event );
 }
